/*
 * @(#) Main.java 
 *
 * JOTM: Java Open Transaction Manager 
 *
 * This project is developed inside the ObjectWeb Consortium,
 * http://www.objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *
 * Contributor(s): Lutris Technologies, Guillaume Riviere
 *
 *
 * --------------------------------------------------------------------------
 * $Id: Main.java,v 1.9 2002/12/18 10:18:47 jmesnil Exp $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jotm;

import java.io.PrintWriter;
import java.rmi.RemoteException;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;

import org.objectweb.jonas_timer.TimerManager;
import org.objectweb.jonas_timer.TraceTimer;

/**
 * Created on Feb 26, 2002
 * @author  Christophe Ney - Lutris Technologies cney@batisseurs.com
 */
public class Main extends Thread {

    private static final String progname = "JOTM";

    // command line options
    private static Options cmdLineOptions = null;

    // verbose mode
    static boolean verbose = false;
    // debug mode
    static boolean debug = false;
    // default transaction timeout
    static int timeout = 0;
    // is transaction factory remote
    static boolean remote = false;
    // transaction factory URL
    static String factoryName = null;
    // user transaction URL
    static String userTransactionName = null;
    // transaction manager URL
    static String transactionManagerName = null;
    // naming context
    static Context ictx = null;
    // transaction factory
    static TransactionFactory tm = null;
    // logWriter
    static PrintWriter logWriter = new PrintWriter(System.out,true);

    /**
     *  used as shutdown hook
     */
    public void run() {
        System.out.print("shutting down...");
        // unbind from naming as required
        if (ictx != null) {
            if (!remote && (factoryName != null)) {
                try {
                    ictx.unbind(factoryName);
                } catch (NamingException e) {
                    // ignore
                }
            }
            try {
                ictx.unbind(userTransactionName);
            } catch (Exception e) {
                // ignore
            }
            try {
                ictx.unbind(transactionManagerName);
            } catch (Exception e) {
                // ignore
            }
        }
        // unexport remote objects
        try {
            PortableRemoteObject.unexportObject(tm);
        } catch (Exception e) {
            // ignore
        }
        // stop the timer manager
        TimerManager.stop(false);
        logWriter.close();
    }

    public static void printHelp(Options cmdLineOptions) {
        PrintWriter pw = new PrintWriter(System.err, true);
        HelpFormatter hf = new HelpFormatter();
        hf.printHelp("JOTM [options...]", cmdLineOptions);
    }

    private static void verbose(String msg) {
        if (verbose) {
            System.out.println(msg);
        }
    }

    private static void checkRegistryMessage() {
        System.err.println("Current JNDI settings are:");
        boolean found = false;
        try {
            Hashtable env = ictx.getEnvironment();
            for (Enumeration e = env.keys();e.hasMoreElements();) {
                String key = (String)e.nextElement();
                System.err.println("- "+key+"="+env.get(key));
                found = true;
            }
        } catch (NamingException e) {
            // ignore
        }
        if (!found) {
            System.err.println("JNDI properties are not set!");
        } else {
            System.err.println("Check that registry is running on a port matching JNDI properties");
        }
    }

   
	
    public static void main(String[] args) {
	// First, we put the Carol jvm arguments,
	// for the java -jar jvm lauching
	System.setProperty("javax.rmi.CORBA.PortableRemoteObjectClass", "org.objectweb.carol.rmi.multi.MultiPRODelegate");
	System.setProperty("java.naming.factory.initial", "org.objectweb.carol.jndi.spi.MultiOrbInitialContextFactory");
        
        cmdLineOptions = new Options();
        // option parameters are: short description (char), long description (String), has arguments (boolean), 
        //                        description (String), required (boolean), has multiple arguments (boolean)
        cmdLineOptions.addOption('d', "debug", false, "debug mode", false, false);        
        cmdLineOptions.addOption('v', "verbose", false, "verbose mode", false, false);
        cmdLineOptions.addOption('f', "factory", true, "JNDI URL of the transaction factory", false, false);        
        cmdLineOptions.addOption('h', "help", false, "print this message and exit", false, false);
        cmdLineOptions.addOption('m', "transaction_manager", true, 
                                 "JNDI URL of the TransactionManager", false, false);
        cmdLineOptions.addOption('r', "remote", false, "lookup remote transaction factory", false, false);        
        cmdLineOptions.addOption('t', "timeout", true, "default transaction timeout (in seconds)", false, false);
        cmdLineOptions.addOption('u', "user_transaction", true, 
                                 "JNDI URL of the UserTransaction", false, false); 
        
        CommandLine cmd = null;
        try {
            cmd = cmdLineOptions.parse(args, true);
        } catch (ParseException e) {
            System.err.println("\n"+ e.getMessage());
            printHelp(cmdLineOptions);
            System.err.println();
            System.exit(1);
        }
        
        debug = cmd.hasOption('d');
        remote = cmd.hasOption('r');
        verbose = cmd.hasOption('v');
        if (cmd.hasOption('h')) {
            printHelp(cmdLineOptions);
            System.exit(1);
        }
        if (cmd.hasOption('f')) {
            factoryName = cmd.getOptionValue('f');
        }
        if (remote && (factoryName == null)) {
                System.err.println("\nfactory option required with remote one.");
                printHelp(cmdLineOptions);
                System.err.println();
                System.exit(1);                
        }
        if (cmd.hasOption('m')) {
            transactionManagerName = cmd.getOptionValue('m');
        }
        if (cmd.hasOption('t')) {
            try {
                timeout = Integer.parseInt(cmd.getOptionValue('t'));
            } catch (NumberFormatException e) {
                System.err.println("\ntimeout is not a number");
                printHelp(cmdLineOptions);
                System.err.println();
                System.exit(1);                
            }
        }
        if (cmd.hasOption('u')) {
            userTransactionName = cmd.getOptionValue('u');
        }

        verbose("UserTransaction Name =" + userTransactionName);
        verbose("TransactionManager Name =" + transactionManagerName);
        verbose("Transaction factory =" + (remote?"remote":"local"));
        verbose("Transaction factory Name =" + factoryName);
        verbose("Default transaction timeout =" + timeout);


        TraceTimer.setLogWriter(logWriter);
        TraceTimer.setVerbose(verbose);
        TraceTimer.setDebug(debug);
		TraceTm.configure();

        try {
            ictx = new InitialContext();
        } catch (NamingException e) {
            System.err.println("No initial context: " + e.getExplanation());
            e.printStackTrace();
            System.exit(1);
        }

        // transaction factory
        if (remote) {
            try {
                tm = (TransactionFactory) ictx.lookup(factoryName);
            } catch (NamingException e) {
                System.err.println("Transaction factory lookup failed : " + e.getExplanation());
                System.exit(1);
            }
        } else {
            try {
                tm = new TransactionFactoryImpl();
            } catch (RemoteException e) {
                System.err.println("Instanciation of transaction factory failed : " + e.getMessage());
                System.exit(1);
            }
            try {
                if (factoryName != null) {
                    ictx.rebind(factoryName, tm);
                    System.out.println("TransactionFactory object bound in JNDI with name "+ factoryName);
                }
            } catch (NamingException e) {
                System.err.println("Transaction factory rebind failed : " + e.getExplanation());
                checkRegistryMessage();
                System.exit(1);
            }
        }

        // create Current
        Current current = new Current(tm);

        // trap program termination
        Runtime.getRuntime().addShutdownHook(new Main());

        try {
            // current implements javax.transaction.UserTransaction
            if (userTransactionName!=null) {
                ictx.rebind(userTransactionName,current);
                System.out.println("UserTransaction object bound in JNDI with name "+userTransactionName);
            }
            // current implements javax.transaction.TransactionManager
            // current implements org.objectweb.transaction.TransactionManager
            if (transactionManagerName!=null) {
                ictx.rebind(transactionManagerName,current);
                System.out.println("TransactionManager object bound in JNDI with name "+transactionManagerName);
            }
        } catch (NamingException e) {
            System.err.println("UserTransaction rebind failed :" + e.getExplanation());
            e.printStackTrace();
            checkRegistryMessage();
            System.exit(1);
        }

        System.out.print(progname+" is running...");

    }

}
